##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_mld_api.tcl
#
# Purpose:
#    A script development library containing MLD APIs for test automation with
#    the Ixia chassis.
#
# Author:
#    Ixia engineering, direct all communication to support@ixiacom.com
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_mld_config
#    - emulation_mld_group_config
#    - emulation_mld_control
#
# Requirements:
#    ixiaapiutils.tcl , a library containing TCL utilities
#    parseddashedargs.tcl , a library containing the procDescr and
#    parsedashedargds.tcl
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

proc ::ixia::emulation_mld_config { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_mld_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable mld_host_handles_array
    variable mld_group_handles_array
    variable mld_attributes_array

    ::ixia::utrackerLog $procName $args

    set man_args {
        -mode CHOICES create modify delete disable enable enable_all disable_all
    }

    set opt_args {
        -atm_encapsulation           CHOICES VccMuxIPV4Routed VccMuxIPV6Routed 
                                     CHOICES VccMuxBridgedEthernetFCS 
                                     CHOICES VccMuxBridgedEthernetNoFCS 
                                     CHOICES LLCRoutedCLIP LLCBridgedEthernetFCS
                                     CHOICES LLCBridgedEthernetNoFCS 
                                     DEFAULT LLCBridgedEthernetFCS
        -count                       RANGE   1-4000
                                     DEFAULT 1
        -enable_packing              CHOICES 0 1 
                                     DEFAULT 0
        -filter_mode                 CHOICES include exclude
                                     DEFAULT include
        -general_query               CHOICES 0 1
                                     DEFAULT 1
        -group_query                 CHOICES 0 1
                                     DEFAULT 1
        -handle
        -interface_handle
        -intf_ip_addr                IPV6
        -intf_ip_addr_step           IPV6
                                     DEFAULT 0::1
        -intf_prefix_len             RANGE   1-128 
                                     DEFAULT 64
        -ip_router_alert             CHOICES 0 1
                                     DEFAULT 1
        -mac_address_init            MAC
        -mac_address_step            MAC
                                     DEFAULT 0000.0000.0001
        -max_groups_per_pkts         RANGE   0-1500
                                     DEFAULT 0
        -max_response_control        CHOICES 0 1
                                     DEFAULT 0
        -max_response_time           RANGE 0-999999
        -max_sources_per_group       RANGE   0-1500 
                                     DEFAULT 0
        -mldv2_report_type           CHOICES 143 206
                                     DEFAULT 143
        -mld_version                 CHOICES v1 v2
                                     DEFAULT v2
        -msg_count_per_interval      RANGE   0-999999999 
                                     DEFAULT 0
        -msg_interval                RANGE   0-999999999
                                     DEFAULT 0
        -neighbor_intf_ip_addr       IPV6 
                                     DEFAULT 0::0
        -neighbor_intf_ip_addr_step  IPV6 
                                     DEFAULT 0::0
        -no_write                    FLAG
        -override_existence_check    CHOICES 0 1
                                     DEFAULT 0
        -override_tracking           CHOICES 0 1
                                     DEFAULT 0
        -port_handle                 REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -reset                       FLAG
        -robustness                  RANGE 1-65535
                                     DEFAULT 2
        -suppress_report             CHOICES 0 1
                                     DEFAULT 0
        -unsolicited_report_interval RANGE   0-999999
        -vci                         RANGE 0-65535
                                     DEFAULT 32
        -vci_step                    RANGE 0-65535
                                     DEFAULT 1
        -vlan                        CHOICES 0 1
        -vlan_cfi                    CHOICES 0 1
        -vlan_id                     RANGE   0-4095
        -vlan_id_mode                CHOICES fixed increment 
                                     DEFAULT increment
        -vlan_id_step                RANGE   0-4095
                                     DEFAULT 1
        -vlan_user_priority          RANGE   0-7
                                     DEFAULT 0
        -vpi                         RANGE 0-255
                                     DEFAULT 1
        -vpi_step                    RANGE 0-255
                                     DEFAULT 1
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_mld_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_mld_group_config { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api
    variable mld_port
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_mld_group_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -mode           CHOICES create modify delete clear_all
    }

    set opt_args {
        -g_enable_packing        CHOICES  0 1
        -g_filter_mode           CHOICES  include exclude
        -g_max_groups_per_pkts   NUMERIC
        -g_max_sources_per_group RANGE    0-1500
        -group_pool_handle
        -handle
        -no_write                FLAG
        -reset                   FLAG
        -session_handle
        -source_pool_handle
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_mld_group_config $args $man_args \
                $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_mld_control { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_mld_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode   CHOICES start stop restart join leave
    }

    set opt_args {
        -handle
        -port_handle          REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -group_member_handle
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_mld_control $args $man_args \
                $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}
